Index: Makefile
===================================================================
--- Makefile	(revision 19250)
+++ Makefile	(working copy)
@@ -727,6 +727,7 @@
 		echo ";lightbackground = yes ; If your terminal is set for a light-colored background" ; \
 		echo "documentation_language = en_US ; Set the Language you want Documentation displayed in. Value is in the same format as locale names" ; \
 		echo ";hideconnect = yes ; Hide messages displayed when a remote console connects and disconnects" ; \
+		echo ";lockconfdir = no ; Protect the directory containing the configuration files (/etc/asterisk) with a lock" ; \
 		echo "" ; \
 		echo "; Changing the following lines may compromise your security." ; \
 		echo ";[files]" ; \
Index: include/asterisk/options.h
===================================================================
--- include/asterisk/options.h	(revision 19250)
+++ include/asterisk/options.h	(working copy)
@@ -90,6 +90,8 @@
 	AST_OPT_FLAG_FORCE_BLACK_BACKGROUND = (1 << 27),
 	/*! Hide remote console connect messages on console */
 	AST_OPT_FLAG_HIDE_CONSOLE_CONNECT = (1 << 28),
+	/*! Protect the configuration file path with a lock */
+	AST_OPT_FLAG_LOCK_CONFIG_DIR = (1 << 29),
 };
 
 /*! These are the options that set by default when Asterisk starts */
@@ -122,6 +124,7 @@
 #define ast_opt_light_background		ast_test_flag(&ast_options, AST_OPT_FLAG_LIGHT_BACKGROUND)
 #define ast_opt_force_black_background		ast_test_flag(&ast_options, AST_OPT_FLAG_FORCE_BLACK_BACKGROUND)
 #define ast_opt_hide_connect		ast_test_flag(&ast_options, AST_OPT_FLAG_HIDE_CONSOLE_CONNECT)
+#define ast_opt_lock_confdir		ast_test_flag(&ast_options, AST_OPT_FLAG_LOCK_CONFIG_DIR)
 
 extern struct ast_flags ast_options;
 
Index: main/loader.c
===================================================================
--- main/loader.c	(revision 19250)
+++ main/loader.c	(working copy)
@@ -50,6 +50,7 @@
 #include "asterisk/dsp.h"
 #include "asterisk/udptl.h"
 #include "asterisk/heap.h"
+#include "asterisk/app.h"
 
 #include <dlfcn.h>
 
@@ -625,6 +626,7 @@
 {
 	struct ast_module *cur;
 	int res = 0; /* return value. 0 = not found, others, see below */
+	int try, ret;
 	int i;
 
 	/* If we aren't fully booted, we just pretend we reloaded but we queue this
@@ -640,6 +642,20 @@
 	}
 	ast_lastreloadtime = ast_tvnow();
 
+	if (ast_opt_lock_confdir) {
+	        for (try = 1, ret = AST_LOCK_TIMEOUT; try < 6 && (ret == AST_LOCK_TIMEOUT); try++) {
+			ret = ast_lock_path(ast_config_AST_CONFIG_DIR);
+			if (ret == AST_LOCK_TIMEOUT) {
+				ast_log(LOG_WARNING, "Failed to grab lock on %s, try %d \n", ast_config_AST_CONFIG_DIR, try);
+			}
+	        }
+		if (ret != AST_LOCK_SUCCESS) {
+			ast_verbose("Cannot grab lock on %s\n", ast_config_AST_CONFIG_DIR);
+			ast_mutex_unlock(&reloadlock);
+			return -1;
+		}
+	}
+
 	/* Call "predefined" reload here first */
 	for (i = 0; reload_classes[i].name; i++) {
 		if (!name || !strcasecmp(name, reload_classes[i].name)) {
@@ -649,6 +665,9 @@
 	}
 
 	if (name && res) {
+		if (ast_opt_lock_confdir) {
+			ast_unlock_path(ast_config_AST_CONFIG_DIR);
+		}
 		ast_mutex_unlock(&reloadlock);
 		return res;
 	}
@@ -683,6 +702,9 @@
 	}
 	AST_LIST_UNLOCK(&module_list);
 
+	if (ast_opt_lock_confdir) {
+		ast_unlock_path(ast_config_AST_CONFIG_DIR);
+	}
 	ast_mutex_unlock(&reloadlock);
 
 	return res;
